--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local addonName, NS = ...

local Lib = NS.FizzleLib

-------------------------- Default Button -------------------------------------------------
function Lib:CreateDefaultButton(name, parent, width, height)
	local f = CreateFrame("Button", "$parent"..name, parent, "UIPanelButtonTemplate")
	f:SetSize(width, height)
	function f:SetLabel(label)
		self:SetText(label)
	end
	return f
end

------------------ Default List Button (Drop/Scroll) --------------------------------------
function Lib.CreateDefaultListButton(button)
	button:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	button.Text = Lib:CreateFont("Text", button, id)
--	button.Text:SetPoint("TOPLEFT", 8, -1)
--	button.Text:SetPoint("BOTTOMRIGHT", -8, 1)

	button.Text:SetPoint("LEFT", 5, 0)

	button.Text:SetShadowColor(0, 0, 0, 1)
	button.Text:SetShadowOffset(1, -1)
	button.Selected = button:CreateTexture()
	button.Selected:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
	button.Selected:SetAllPoints()
	button.Selected:Hide()
	button:HookScript("OnClick", function(self)
--		if self:GetParent().LastSelected then
--			self:GetParent().LastSelected:Hide()
--		end
		if self:GetParent().LastSelected then
			self:GetParent().LastSelected:Hide()
		end
		self:GetParent().LastIndex = self.Index
		self:GetParent().LastSelected = self.Selected
		self.Selected:Show()
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	end)
end

--------------------------- Close Button --------------------------------------------------
function Lib:CreateCloseButton(name, parent, width, height, colour)
	if type(width) == "string" then
		colour = width
		width = nil
	end
	if type(height) == "string" then
		colour = height
		height = nil
	end

	local w = width or 24
	local h = height or 24
	local f = CreateFrame("Button", "$parent"..name, parent)--, "UIPanelCloseButton")
	f:SetSize(w, h)
	f:SetPoint("TOPRIGHT", parent, "TOPRIGHT")--, -2, -4)

	if colour and type(colour) == "string" then
		f:SetNormalTexture(Lib.TexturePath.."Close"..colour)
		f:SetPushedTexture(Lib.TexturePath.."ClosePushed"..colour)
		f:SetHighlightTexture(Lib.TexturePath.."CloseOverlay", "ADD")
	else
		f:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		f:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		f:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	end
	f:SetScript("OnClick", function(self)
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			parent:Hide()
		end)

	return f
end

---------------------------Colour Picker Button--------------------------------------------
local function SetPickerColour(self, r, g, b)
	if self.LED then
		self.LED:SetVertexColor(r, g, b)
	else
		self:SetBackdropColor(r, g, b)
	end
end

local function ColourPicker_Init(self)
	local colour = self.Data[self.Index]
	if not colour then
		colour = {r=1, g=1, b=1}
	end
	SetPickerColour(self, colour.r, colour.g, colour.b)
end

local function ColourPicker_Cancelled(self)
	local colour = ColorPickerFrame.previousValues
	ColorPickerFrame.colourBox.Data[ColorPickerFrame.colourBox.Index] = colour
	SetPickerColour(ColorPickerFrame.colourBox, colour.r, colour.g, colour.b)
	if ColorPickerFrame.colourBox.PostUpdate then
		ColorPickerFrame.colourBox:PostUpdate()
	end
end

local function ColourPicker_Changed(self)
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local colour = { r=r, g=g, b=b }
	ColorPickerFrame.colourBox.Data[ColorPickerFrame.colourBox.Index] = colour
	SetPickerColour(ColorPickerFrame.colourBox, colour.r, colour.g, colour.b)
	if ColorPickerFrame.colourBox.PostUpdate then
		ColorPickerFrame.colourBox:PostUpdate()
	end
end

local function ColourPicker_OnClick(self)
	local basecolour = self.Data[self.Index]
	local colour = {}
	colour.r = basecolour.r
	colour.g = basecolour.g
	colour.b = basecolour.b
	ColorPickerFrame.previousValues = colour
	ColorPickerFrame.colourBox = self
	if ColorPickerFrame.SetupColorPickerAndShow then  -- 10.2.5 fix ColorSelect widget moved from top level to "Frame".Content.ColorPicker
		local info = UIDropDownMenu_CreateInfo()
		info.r, info.g, info.b = colour.r, colour.g, colour.b
		info.swatchFunc = ColourPicker_Changed
		info.cancelFunc = ColourPicker_Cancelled
		ColorPickerFrame:SetupColorPickerAndShow(info)
		if ColorPickerFrame.Content then
			ColorPickerFrame.Content.ColorPicker:SetColorRGB(colour.r, colour.g, colour.b)
		else
			ColorPickerFrame:SetColorRGB(colour.r, colour.g, colour.b)
		end
	else
		ColorPickerFrame.cancelFunc = ColourPicker_Cancelled
--		ColorPickerFrame.opacityFunc = ColourPicker_Changed
		ColorPickerFrame.func = ColourPicker_Changed
		ColorPickerFrame:SetColorRGB(colour.r, colour.g, colour.b)
	end
	ColorPickerFrame:ClearAllPoints()
	if self:GetRight() < UIParent:GetWidth() / 2 then
		ColorPickerFrame:SetPoint("LEFT", self, "RIGHT", 10, 0)
	else
		ColorPickerFrame:SetPoint("RIGHT", self, "LEFT", -10, 0)
	end
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	ColorPickerFrame:Show()
end

local function SetColourPickerOnClick(self)
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	self:SetScript("OnClick", function(self, button, down)
		if button == "RightButton" then
			if self.DefaultColor then
				SetPickerColour(self, self.DefaultColor.r, self.DefaultColor.g, self.DefaultColor.b)
				self.Data[self.Index].r = self.DefaultColor.r
				self.Data[self.Index].g = self.DefaultColor.g
				self.Data[self.Index].b = self.DefaultColor.b
				ColourPicker_Init(self)
				if self.PostUpdate then
					self:PostUpdate()
				end
			end
			return
		end
		ColourPicker_OnClick(self, button, down)
	end)
end
function Lib:CreateColorPickerButton(name, parent, data, index)
	local f = Lib:CreateTooltipBackdrop(name, parent, "Button")
	f:SetSize(24, 24)
	Lib:AddLabelFuncs(f)
	Lib.InitData(f, data, index, ColourPicker_Init)
	SetColourPickerOnClick(f)
	return f
end

function Lib:CreateLEDColorPickerButton(name, parent, data, index)
	local f = CreateFrame("Button", "$parent"..name, parent)
	f:SetSize(24, 24)
	f.LED = f:CreateTexture(nil)
	f.LED:SetDrawLayer("BACKGROUND", 1)
	f.LED:SetAllPoints()
	f.LED:SetTexture(Lib.TexturePath.."LEDCircleWhite")
	f.Glass = f:CreateTexture(nil)
	f.Glass:SetDrawLayer("BACKGROUND", 4)
	f.Glass:SetAllPoints()
	f.Glass:SetTexture(Lib.TexturePath.."LEDGlass")
--	f.Glass:SetVertexColor(1, 1, 1, 0.5)
	f.Glass:SetAlpha(0.6)
	f.Overlay = f:CreateTexture(nil)
	f.Overlay:SetAllPoints()
	f.Overlay:SetDrawLayer("OVERLAY", 1)
	f.Overlay:SetTexture(Lib.TexturePath.."LEDOverlayRing")
	f.Brush = f:CreateTexture(nil)
	f.Brush:SetSize(18, 18)
	f.Brush:SetDrawLayer("OVERLAY", 4)
	f.Brush:SetPoint("BOTTOMLEFT", f, "CENTER", -4, -4)
	f.Brush:SetTexture(Lib.TexturePath.."ColourBrush")
	hooksecurefunc(f, "SetEnabled", Lib.SetEnabled)
	Lib:AddLabelFuncs(f)
	Lib.InitData(f, data, index, ColourPicker_Init)
	SetColourPickerOnClick(f)
	return f
end

--------------------------- Check Buttons -------------------------------------------------
local function CheckButton_Init(self) --, data, index)
	self:SetChecked(self.Data[self.Index])
end

function Lib:CreateBaseCheckButton(name, parent, data, index, template)
	local f
	if type(name) == "table" then
		f = name
	else
		f = CreateFrame("CheckButton", "$parent"..name, parent, template)-- ButtonCreate(name, parent)
	end
	f:SetSize(22, 22)
--	Lib.InitData(f, data, index, CheckButton_Init)
	f:HookScript("OnClick", function(self)
		local checked = self:GetChecked()
		self.Data[self.Index] = checked
		PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
		if self.PostUpdate then -- Yes, this is needed
			self:PostUpdate()--self, self.Data, self.Index)
		end
	end)
	Lib:AddLabelFuncs(f)
	return f
end

function Lib:CreateCheckButton(name, parent, data, index)
	local f = Lib:CreateBaseCheckButton(name, parent, data, index, "UICheckButtonTemplate")
	Lib.InitData(f, data, index, CheckButton_Init)
	return f
end

function Lib:CreateCustomCheckButton(name, parent, data, index, unchecked, checked, disabled, disabledchecked)
	local enabled = true
	if type(unchecked) == "boolean" then
		enabled = unchecked
		unchecked = nil
	elseif type(checked) == "boolean" then
		enabled = checked
		checked = nil
	elseif type(disabled) == "boolean" then
		enabled = disabled
		disabled = nil
	elseif type(disabledchecked) == "boolean" then
		enabled = disabledchecked
		disabledchecked = nil
	end
	local f = Lib:CreateBaseCheckButton(name, parent, data, index)
	if unchecked then
		f:SetNormalTexture(unchecked)
	end
	if checked then
		f:SetCheckedTexture(checked)
	end
	if disbled then
		f:SetDisabledTexture(disabled)
	end
	if disabledchecked then
		f:SetDisabledCheckedTexture(disabledchecked)
	end
	Lib.InitData(f, data, index, CheckButton_Init)
	return f
end

function Lib:CreateLedCheckButton(name, parent, data, index, hot, cold)
	local f = Lib:CreateBaseCheckButton(name, parent, data, index)
	hot = hot or "Green"
	if cold then
		f:SetNormalTexture(Lib.TexturePath.."LED"..cold)
	else
		f:SetNormalTexture(Lib.TexturePath.."LEDBlack")
	end
	f:SetCheckedTexture(Lib.TexturePath.."LED"..hot)
	f:SetDisabledCheckedTexture(Lib.TexturePath.."LEDDisabled")
	f:SetDisabledTexture(Lib.TexturePath.."LEDDisabled")
	local t = f:GetCheckedTexture()
	t:SetBlendMode("BLEND")
	t = f:GetDisabledCheckedTexture()
	t:SetBlendMode("BLEND")
	t = f:GetDisabledTexture()
	t:SetBlendMode("BLEND")
	Lib.InitData(f, data, index, CheckButton_Init)
	return f
end

------------------------ Multi State Button ---------------------------------------------
function Lib:CreateLedMultiStateButton(name, parent, data, index, defaultstate)
	local f = CreateFrame("Button", "$parent"..name, parent)
	local enabled = true
	f:SetSize(18, 18)
	f.States = { }
	f.Texture = f:CreateTexture("$parentStateTexture")
	f.Texture:SetAllPoints()
	f.States[0] = { t=defaultstate, r=1, g=1, b=1, a=1 }
	function f.AddState(self, texture, val, r, g, b, a)
		local next = #self.States + 1
		self.States[next] = { t=texture, v=val, r=r or 1, g=g or 1, b=b or 1, a=a or 1 }
		return next
	end
	function f.GetState(self)
		return self.States[self.ActionState].v
	end
	function f.IncrementState(self)
		local state = self.ActionState + 1
		if state > #self.States then
			state = 0
		end
		self:SetState(state)
	end
	function f.SetState(self, state)
		local count = #self.States
		if state > count then
			state = count
		end
		if state < 0 then
			state = 0
		end
		local settings = self.States[state]
		self.Texture:SetTexture(settings.t)
		self.Texture:SetVertexColor(settings.r, settings.g, settings.b, settings.a)
		self.ActionState = state
	end
	function f.SetStateColor(self, state, r, g, b, a)
		local settings = self.States[state]
		settings.r = r
		settings.g = g
		settings.b = b
		settings.a = a or 1
		if self.ActionState == state then
			self.Texture:SetVertexColor(settings.r, settings.g, settings.b, settings.a)
		end
	end
	f:SetScript("OnClick", function(self)
			self:IncrementState()
			PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
			if self.clickfunc then
				self:clickfunc(self.States[self.ActionState].v)
			end
		end)
	f:SetState(0)
	return f
end

------------------------ Radio Buttons ---------------------------------------------

function Lib:CreateRadioGroup(name, parent, data, index, backdrop, radiotemplate)
	local template-- = "UIRadioButtonTemplate"
	if backdrop and type(backdrop) == "string" then
		template = backdrop
		backdrop = nil
	end
	local name = backdrop and name or "$parent"..name
	local f = backdrop and backdrop(name, parent, "Frame") or CreateFrame("Frame", name, parent)
	f:SetSize(5, 5)
	f.Buttons = {}
	for i=1, 2 do
		if not template then
			f.Buttons[i] = CreateFrame("CheckButton", "$parentButton"..1, f, "UIRadioButtonTemplate")
		else
		end
		f.Buttons[i]:SetPoint("TOPLEFT", f.Buttons[i-1] or f, f.Buttons[i-1] and "TOPRIGHT" or "TOPLEFT", f.Buttons[i-1] and 3 or 0, f.Buttons[i-1] and -3 or 0)
	end
	return f
end

------------------------ LED Buttons ---------------------------------------------

function Lib:CreateLED(name, parent, ftype, initialcolor, shape)
	initialcolor = initialcolorcolor or "Green"
	shape = shape or "Circle"
	local f = CreateFrame(ftype, "$parent"..name, parent)
	f:SetSize(5, 5)
	f.Color = initialcolor
	f.Shape = shape
	
	f.BG = f:CreateTexture(nil)
	f.BG:SetDrawLayer("BACKGROUND", -4)
	f.BG:SetAllPoints()
	f.BG:SetTexture(Lib.TexturePath.."LED"..shape.."White")

	f.LED = f:CreateTexture(nil)
	f.LED:SetDrawLayer("BACKGROUND", 0)
	f.LED:SetAllPoints()
	f.LED:SetTexture(Lib.TexturePath.."LED"..shape..initialcolor)

	f.Overlay = f:CreateTexture(nil)
	f.Overlay:SetAllPoints()
	f.Overlay:SetDrawLayer("OVERLAY", 1)
	f.Overlay:SetTexture(Lib.TexturePath.."LED"..shape.."Overlay")
	
	hooksecurefunc(f, "SetEnabled", function(self, state)
		if not state then
			self.LED:SetTexture(Lib.TexturePath.."LED"..shape.."Black")
		else
			self.LED:SetTexture(Lib.TexturePath.."LED"..shape..initialcolor)
		end
	end)
	return f
end
